/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.software.SoftwareCatagoryForm;
import rjw.dc.core.objects.DataCrowCollection;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.db.QueryOptions;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;


/**
 * Base for all search panels (for objects like Software, Mp3 and Movie)
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public abstract class TableSearchPanel extends TablePanel {

    protected SearchOptionsPanel panelSearchOpts;
    protected SearchConditionsPanel panelSearchConds;
    protected JPanel panelStatus;
    protected JPanel panelResult;
    protected JPanel panelSearchActs;
    protected JScrollPane scrollSearchConds;
    
    protected JButton buttonSearch;

    /**
     * Initializes the panel. Should be called by the child by calling
     * super(module). The popup menu and the action panel are automatically
     * created.
     *
     * @param module the module indentifier
     */
    public TableSearchPanel(int module) {
        super();
        this.module = module;

        DataCrowObject o = WorkFlow.getObject(module);
        boolean imdb = o.isImdbUpdateAllowed();
        boolean amazon = o.isAmazonUpdateAllowed();
        boolean open = this instanceof SoftwareCatagoryForm.CatagorySearchPanel ?
                       false : true;

        panelSearchOpts = new SearchOptionsPanel();
        panelSearchConds = new SearchConditionsPanel();

        panelTableActions = new TableActionsPanel(false);
        panelTableActions.addSaveActionListener(new SaveUpdatedValuesAction());
    
        popupMenu = new PopupMenu(imdb, amazon, true, false, open, true);
        popupMenu.addAmazonActionListener(new AmazonUpdateAction());
        popupMenu.addIMDBActionListener(new ImdbUpdateAction());
        popupMenu.addSaveActionListener(new SaveSelectedValuesAction());
        popupMenu.addDeleteActionListener(new DeleteValuesAction());
        
        buildPanel();
        
        for (Enumeration enum = table.getColumnModel().getColumns();
             enum.hasMoreElements(); ) {
            
            vColumns.addElement(enum.nextElement());
        }
    }

    /**
     * Saves the selected values
     */
    public void saveSelectedValues() {
        cancelEdit();
        int[] rows = table.getSelectedRows();
        DataCrowCollection collection = new DataCrowCollection();
        if (rows != null && rows.length > 0) {
            DataCrowObject o = WorkFlow.getObject(module);
            for (int i = 0; i < rows.length; i++) {
                Object[] row = getValuesAtRow(rows[i], true);
                o.setValues(row);
                collection.addObject(o, new Integer(rows[i]));
            }
        }

        RequestorCollection requestors = addUpdateSoftCatRequest(
                                         new RequestorCollection());

        collection.saveUpdate(requestors);
        setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));
    }

    /**
     * Saves all the changed values
     */
    public void saveChangedValues() {
        cancelEdit();
        DataCrowCollection collection = new DataCrowCollection();

        for (Enumeration enum = vChangedIDs.elements(); enum.hasMoreElements(); ) {
            int rowNumber = getRowNumberWithID(enum.nextElement().toString());
            Object[] row = getValuesAtRow(rowNumber, true);
            DataCrowObject o = WorkFlow.getObject(module);
            o.setValues(row);
            collection.addObject(o, new Integer(rowNumber));
        }

        RequestorCollection requestors = addUpdateSoftCatRequest(
                                         new RequestorCollection());

        collection.saveUpdate(requestors);
        vChangedIDs.removeAllElements();
        setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));
    }

    /**
     * Searches for the Software objects by using the conditions and options
     * as specified.
     */
    public void searchValues() {
        cancelEdit();
        DataCrowObject o = WorkFlow.getObject(module);
        o.setValues(panelSearchConds.getSearchValues());

        tableModel.removeTableModelListener(new TableValueChangedAction());

        Requestor requestor = new Requestor(this.getClass(), "fillTable", null);
        RequestorCollection requestors = new RequestorCollection();
        requestors.addRequestor(requestor);

        o.load(panelSearchOpts.getQueryOptions(), requestors);
        setStatus(Resources.getText(DCMessages.msgSearchHasBeenExecuted));

        tableModel.addTableModelListener(new TableValueChangedAction());
        
        vChangedIDs.removeAllElements();
    }
    
    public void deleteValues() {
        cancelEdit();
        int[] rows = table.getSelectedRows();

        if (rows == null || rows.length <= 0) {
            new MessageBox(Resources.getText(DCMessages.msgNoRowsSelected),
                           true, MessageBox._NOTIFICATION);
            return;
        }

        DataCrowCollection collection = new DataCrowCollection();

        for (int i = 0; i < rows.length; i++) {
            DataCrowObject o = WorkFlow.getObject(module);
            o.setValue(DataCrowObject._ID, tableModel.getValueAt(rows[i], 0));
            collection.addObject(o, new Integer(rows[i]));
        }

        GeneralDialog dialog = new GeneralDialog(Resources.getText(DCMessages.msgDeleteQuestion));

        if (((Boolean) GeneralDialog.result).booleanValue()) {
            RequestorCollection requestors = new RequestorCollection();
            Requestor request = new Requestor(getClass(), "removeRows",
                                              new Object[] {table.getSelectedRows()});

            requestors = addUpdateSoftCatRequest(requestors);

            requestors.addRequestor(request);
            collection.delete(requestors);

            setStatus(Resources.getText(DCMessages.msgDeleteHasBeenExecuted));
        }
    }
    
    public void removeHiddenFields(int[] indices) {
        panelSearchConds.removeHiddenFields(indices);
        super.removeHiddenFields(indices);  
    }
    
    public class SearchConditionsPanel extends JPanel {
        private DataCrowObject o;
        
        private final Vector vInput = new Vector(0);
        private final Vector vLabels = new Vector(0);
        
        public SearchConditionsPanel() {
            o = WorkFlow.getObject(module);
            buildPanel(); 
            removeHiddenFields(o.getHiddenFields());  
        }
        
        private void buildPanel() {
            String[] labels = o.getLabels();
            int[] fields = o.getFieldIndices();
            int[] maxLengths = o.getMaxFieldLengths();
            int[] fieldTypes = o.getFieldTypes();

            setLayout(new GridBagLayout());
            int y = 0;
            for (int i = 0; i < fields.length; i++) {
                int field = fields[i];
                JLabel label = ComponentFactory.getLabel(labels[field]);
                
                JComponent input = null;
                if (fieldTypes[field] == ComponentFactory._LONGTEXTFIELD) {
                    input = ComponentFactory.getComponent(ComponentFactory._SHORTTEXTFIELD, 
                                                          labels[field],
                                                          maxLengths[field]); 
                } else {
                    input = ComponentFactory.getComponent(fieldTypes[field], 
                                                          labels[field],
                                                          maxLengths[field]); 
                }

                int bottom = 0;
                if (i == fields.length -1) {
                    bottom = 5;   
                }
    
                add( label, new GridBagConstraints( 0, y, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0,  5, bottom, 10), 0, 0));
                add( input, new GridBagConstraints( 1, y, 1, 1, 5.0, 5.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 10, bottom,  5), 0, 0));

                vLabels.addElement(label);
                vInput.addElement(input);
                y++;
            }
        }

        public void removeHiddenFields(int[] indices) {
            o = WorkFlow.getObject(module);
            int[] visibleFields = o.getVisibleFields();

            for (int i = 0; i < visibleFields.length; i++) {
                JComponent comp = (JComponent) vInput.elementAt(visibleFields[i]);
                comp.setVisible(true);
                JLabel label = (JLabel) vLabels.elementAt(visibleFields[i]);
                label.setVisible(true);
            }

            for (int i = 0; i < indices.length; i++) {
                JComponent comp = (JComponent) vInput.elementAt(indices[i]);
                comp.setVisible(false);
                JLabel label = (JLabel) vLabels.elementAt(indices[i]);
                label.setVisible(false);
            }
        }
                
        public Object[] getSearchValues() {
            o = WorkFlow.getObject(module);
            final Object[] values = o.getValues();
            int[] visibleFields = o.getVisibleFields();
            int[] fieldTypes = o.getFieldTypes();

            for (int i = 0; i < visibleFields.length; i++) {
                int fieldType = fieldTypes[visibleFields[i]];
                JComponent component = (JComponent) vInput.elementAt(visibleFields[i]);
                
                if (fieldType == ComponentFactory._LONGTEXTFIELD) {
                    values[visibleFields[i]] = ComponentFactory.getComponentValue(
                                                    ComponentFactory._SHORTTEXTFIELD,
                                                    component);    
                } else {
                    values[visibleFields[i]] = ComponentFactory.getComponentValue(
                                                    fieldType,
                                                    component);    
                }
            }
            
            for (int i = 0; i < values.length; i++) {
                if (values[i] != null && values[i] instanceof Integer) {
                    if (((Integer) values[i]).intValue() == 0) {
                        values[i] = new Integer(-1); 
                    }
                }
            }
            
            return values;
        }
    }

    public class SearchOptionsPanel extends JPanel {
        private JComboBox comboAndOr;
        private JLabel labelAndOr;
        private JComboBox comboPrecise;
        private JLabel labelPrecise;

        private JLabel labelOrderBy;
        private JComboBox comboOrderBy;

        public SearchOptionsPanel() {
            DataCrowObject o = WorkFlow.getObject(module);

            setLayout(new GridBagLayout());

            // Initialize components
            comboAndOr      = ComponentFactory.getComboBox();
            labelAndOr      = ComponentFactory.getLabel(Resources.getText(DCLabels.lblAndOrSelect));
            comboPrecise    = ComponentFactory.getRealYesNoComboBox();
            labelPrecise    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblPreciseSelect));
            comboOrderBy    = ComponentFactory.getComboBox();
            labelOrderBy    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblOrderBy));

            String[]    colNames    = o.getFieldNames();
            comboOrderBy.addItem("");
            for (int i = 0; i < colNames.length; i++) {
                comboOrderBy.addItem(colNames[i]);
            }

            comboAndOr.addItem(Resources.getText(DCLabels.lblAnd));
            comboAndOr.addItem(Resources.getText(DCLabels.lblOr));

            // Build the panel
            add( labelAndOr,     new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 0, 5), 0, 0));
            add( comboAndOr,     new GridBagConstraints( 1, 0, 1, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 0, 5), 0, 0));
            add( labelPrecise,   new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 0, 5), 0, 0));
            add( comboPrecise,   new GridBagConstraints( 1, 1, 1, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 0, 5), 0, 0));
            add( labelOrderBy,   new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 0, 5), 0, 0));
            add( comboOrderBy,   new GridBagConstraints( 1, 2, 1, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 5, 5), 0, 0));
        }

        /**
         * Returns the query options used for searching values
         *
         * @return QueryOptions
         */
        public QueryOptions getQueryOptions() {
            boolean bPreciseSelect = false;
            boolean bComplyToAllCondition = true;
            String  sYes            = Resources.getText(DCLabels.lblYes);
            String  sOr             = Resources.getText(DCLabels.lblOr);

            if (comboAndOr.getSelectedItem().equals(sOr)) {
                bComplyToAllCondition = false;
            }
            if (comboPrecise.getSelectedItem().equals(sYes)) {
                bPreciseSelect = true;
            }

            String sOrderBy = "";
            if (comboOrderBy.getSelectedItem() != null) {
                sOrderBy    = comboOrderBy.getSelectedItem().toString();
            }

            return new QueryOptions(sOrderBy, bComplyToAllCondition, bPreciseSelect);
        }
    }

    /**
     * Initializes the panel
     */
    private void buildPanel() {
        panelStatus     = new JPanel(false);
        panelResult     = new JPanel(false);
        panelSearchActs = new JPanel(false);

        // ************************************************************************
        // SEARCH ACTION PANEL

        buttonSearch = ComponentFactory.getButton(Resources.getText(DCLabels.lblSearch));
        buttonSearch.setToolTipText(Resources.getText(DCTooltips.tpSearch));
        buttonSearch.addActionListener(new SearchButtonAction());
        buttonSearch.setMnemonic(KeyEvent.VK_E);
 
        panelSearchActs.add( buttonSearch,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets( 0, 5, 0, 5), 0, 0));

        // ************************************************************************
        // RESULT PANEL
        // Set layout style
        GridBagLayout gridbResult   = new GridBagLayout();
        panelResult.setLayout(gridbResult);

        table = ComponentFactory.getTableForObject(WorkFlow.getObject(module), tableModel);
        table.addKeyListener(new TableKeyListener());
        tableModel.addTableModelListener(new TableValueChangedAction());
        table.addMouseListener(new TableMouseListener());

        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelResult.add(    scrollResults,  new GridBagConstraints( 0, 0, 1, 1, 80.0, 80.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));

        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        GridBagLayout gridbStatus       = new GridBagLayout();
        panelStatus.setLayout(gridbStatus);

        // Initialize components
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(    labelStatus,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 0, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbag     = new GridBagLayout();
        setLayout(gridbag);

        // Initialize components
        panelResult.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblSearchResult)));
        panelStatus.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblStatus)));
        panelSearchOpts.setBorder(ComponentFactory.getBorder(
                                  Resources.getText(DCLabels.lblSearchOpts)));

        scrollSearchConds = new JScrollPane(panelSearchConds);
        scrollSearchConds.setBorder(ComponentFactory.getBorder(
                         Resources.getText(DCLabels.lblSearchCond)));

        scrollSearchConds.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scrollSearchConds.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        
        // Build the panel
        add(    scrollSearchConds,new GridBagConstraints( 0, 0, 1, 2, 2.0, 2.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelSearchOpts,  new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelSearchActs,  new GridBagConstraints( 1, 1, 1, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelResult,      new GridBagConstraints( 0, 3, 3, 1, 50.0, 50.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelTableActions,new GridBagConstraints( 0, 4, 3, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelStatus,      new GridBagConstraints( 0, 5, 3, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }

    public void setBackground(Color color) {
        super.setBackground(color);
        
        if (scrollSearchConds != null) {
            scrollSearchConds.setBackground(color);
            panelSearchConds.setBackground(color);
        }
    }

    // ****************************************************************************
    // ACTIONS
    // ****************************************************************************
    public class SearchButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelEdit();
            searchValues();
        }
    }

    private class SaveSelectedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveSelectedValues();
        }
    }

    private class AmazonUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startAmazonUpdate(true);
        }
    }

    private class SaveUpdatedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveChangedValues();
        }
    }

    private class DeleteValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            deleteValues();
        }
    }

    private class ImdbUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelEdit();
            startIMDBUpdate(true);
        }
    }
}
