/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.movie;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import rjw.dc.console.MessageBox;
import rjw.dc.console.SearchForm;
import rjw.dc.core.Repository;
import rjw.dc.http.HttpConnection;
import rjw.dc.http.IMDBQuery;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.util.SwingUtil;
import rjw.flex.settings.Settings;

/**
 * Form used to search for IMDB data. The search is performed on the
 * title of the movie
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ImdbSearchForm extends SearchForm {
    
    /**
     * Instantiates this form and starts the search
     * 
     * @param rowNumber the rownumber of the to be updated record
     * @param title the title for this form
     * @param name the name on which to search
     * @param sender the requestor
     */
    public ImdbSearchForm(int rowNumber, 
                          String title, 
                          String name, 
                          Object sender) {
                              
        super(sender, new int[] {1, 2, 3, 4, 5, 6, 7, 8, 9});
        super.setTitle(title);
        SwingUtil.setRootFrame(this);
        
        isUpdateRunning = true;
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stImdbSearchFormViewSize,
                                  getSize());
                running = false;
            }
        });        
        
        this.rowNumber = rowNumber;
        
        textSearch.setText(name);
        startSearch();
        
        setVisible(true);
    }

    /**
     * Starts the search thread
     */
    protected void startSearch() {
        String name = textSearch.getText();
        
        if (name == null || name.equals("")) {
            return;   
        }
                
        threadSearch = new Thread(new SearchJob(name), "SearchJob");
        threadSearch.setDaemon(true);
        threadSearch.setPriority(1);
        threadSearch.start();        
    }
    
    /**
     * Simple search thread. Uses a query object to contact the
     * internet and puts the results in the table of this form
     * 
     * @author rjwaal
     */
    private class SearchJob extends Thread {
        
        String name = "";
        
        public SearchJob(String name) {
            this.name = name;
        }
        
        public void run() {
            try {
                running = true;
                buttonSearch.setEnabled(false);
                buttonStop.setEnabled(true);
                buttonStopAll.setEnabled(true);
                searchImdb(name);
            } catch (Exception exp) {
                exp.printStackTrace();
                Messages.setMessage(exp.toString(), Messages._ERROR);
                new MessageBox(Resources.getText(DCMessages.msgProcessCrash), 
                               true, MessageBox._ERROR);
            }
        }      

        private void searchImdb(String name) {
            String sHost = Settings.getValueAsString(Repository.settings.stODB);
            HttpConnection connection = null;
            
            try {
                if (running) {
                    connection = new HttpConnection(sHost);
                } else {
                    return;   
                }
            } catch (Exception exp) {
                new MessageBox(exp.getMessage(), true, MessageBox._ERROR);
                Messages.setMessage("<HTTP> " + exp.getMessage(), Messages._ERROR);
                return;
            }
            
            try {
                IMDBQuery imdbQuery = new IMDBQuery(name);
                String[] ids = imdbQuery.getIDs(connection);
                if (ids == null) {
                    return;
                }
                
                progress.setMaximum(ids.length);
                progress.setValue(0);
                
                for (int i = 0; i < ids.length; i++) {
                    if (!running) {
                        return;   
                    }
                    
                    imdbQuery.fillImdbData(ids[i], connection);
                    Object[] row = new Object[10];
                    
                    int j = 0;
                    row[j++] = new Integer(rowNumber);
                    row[j++] = imdbQuery.getValue(IMDBQuery._TITLE);
                    row[j++] = imdbQuery.getValue(IMDBQuery._PLOT);
                    row[j++] = imdbQuery.getValue(IMDBQuery._URL);
                    row[j++] = imdbQuery.getValue(IMDBQuery._GENRE);
                    row[j++] = imdbQuery.getValue(IMDBQuery._ACTORS);
                    row[j++] = imdbQuery.getValue(IMDBQuery._DIRECTOR);
                    row[j++] = imdbQuery.getValue(IMDBQuery._COUNTRY);
                    row[j++] = imdbQuery.getValue(IMDBQuery._YEAR);
                    row[j++] = imdbQuery.getValue(IMDBQuery._RUNTIME);

                    progress.setValue(i + 1);
                    setData(row);
                }
            } catch (Exception exp) {
                Messages.setMessage("<HTTP> " + exp.getMessage(), Messages._ERROR);
                new MessageBox(exp.getMessage(), true, MessageBox._ERROR);   
            } finally {
                buttonSearch.setEnabled(true);
                buttonStop.setEnabled(false);
                buttonStopAll.setEnabled(false);
                connection.close();     
                finishAll();       
            }
        }
        
        /**
         * Stops the process
         */
        private void finishAll() {
            try {
                Messages.setMessage(
                    Resources.getText(DCMessages.msgIMDBProcessStops), 
                    Messages._STANDARD);
            } catch (Exception ignore) {}
        }
    }
}
