/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects.mp3;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Representation of an MP3 file. Allows saving, updating and querying of MP3 records
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Mp3 extends DataCrowObject {

    /** MP3 filename */
    public static final int _A_FILENAME = 1;
    /** Song title */
    public static final int _B_TITLE = 2;
    /** Artist name */
    public static final int _C_ARTIST = 3;
    /** Album name */
    public static final int _D_ALBUM = 4;
    /** Year of creation */
    public static final int _E_YEAR = 5;
    /** Genre name */
    public static final int _F_GENRE = 6;
    /** Play length  */
    public static final int _G_LENGTH = 7;
    /** Sound quality */
    public static final int _H_QUALITY = 8;
    /** Location name (for grouping data) */
    public static final int _I_LOCATION = 9;

    private static WorkFlow wf;

    /**
     * Initializes the MP3 object
     */
    public Mp3() {
       super();
    }

    /**
     * Applies values to this MP3 record. The first in the row is the ID
     *
     * @param row   representation of an database row. First in row is the ID
     */
    public void setValues(Object[] row) {
        int i = 0;
        try {
            if (row.length < Repository.dataBase.dbMP3ColArray.length) {
                throw new Exception(Resources.getText(DCMessages.msgInvalidArray));
            } else if (row.length == Repository.dataBase.dbMP3ColArrayPK.length) {
                if (alwaysGenerateID) {
                    i++; 
                } else {
                    setValue(_ID, row[i++]);
                }
            }

            setValue(_A_FILENAME, row[i++]);
            setValue(_B_TITLE, row[i++]);
            setValue(_C_ARTIST, row[i++]);
            setValue(_D_ALBUM, row[i++]);
            setValue(_E_YEAR, row[i++]);
            setValue(_F_GENRE, row[i++]);

            String length = row[i].toString();
            if (length.indexOf("s") > 0) {
                length = length.substring(0, length.indexOf("s")).trim();
                setValue(_G_LENGTH, length);
                i++;
            } else {
                setValue(_G_LENGTH, row[i++]);
            }
            setValue(_H_QUALITY, row[i++]);
            setValue(_I_LOCATION, row[i++]);

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }
    
    // =========================================================================
    // INITIALIZATION
    // =========================================================================
    protected void setFieldTypes() {
        fieldTypes = Repository.objects.mp3FieldTypes;
    }
    
    protected void setFieldNames() {
        fieldNames = Repository.dataBase.dbMP3ColArrayPK;
    }
    
    protected void setLabels() {
        labels = new String[Repository.dataBase.dbMP3ColArrayPK.length];
        int i = 0;
        labels[i++] = Resources.getText(DCLabels.lblID);
        labels[i++] = Resources.getText(DCLabels.lblFileName);
        labels[i++] = Resources.getText(DCLabels.lblTitle);
        labels[i++] = Resources.getText(DCLabels.lblArtist);
        labels[i++] = Resources.getText(DCLabels.lblAlbum);
        labels[i++] = Resources.getText(DCLabels.lblYear);
        labels[i++] = Resources.getText(DCLabels.lblGenre);
        labels[i++] = Resources.getText(DCLabels.lblPlayLength);
        labels[i++] = Resources.getText(DCLabels.lblQuality);
        labels[i++] = Resources.getText(DCLabels.lblLocation);
    }
    
    protected void setModuleName() {
        moduleName = Resources.getText(DCLabels.lblMp3);    
    }
    
    protected void setTableName() {
        sTableName = Repository.dataBase.dBMP3TableName.trim();
    }
    
    protected void setRequiredFields() {
        requiredFields = Settings.getValueAsIntegerArray(
                                  Repository.settings.stMp3RequiredFields);
    }

    protected void setHiddenFields() {
        hiddenFields = Settings.getValueAsIntegerArray(
                                Repository.settings.stHiddenFieldsMP3);   
    }
    
    protected void setValueTypes() {
        valueTypes = Repository.objects.mp3ValueTypes;
    }
    
    protected void setMaxFieldLengths() {
        maxFieldLength = Repository.objects.mp3FieldLengths;        
    }

    protected void initValues() {
        values = new Object[Repository.dataBase.dbMP3ColArrayPK.length];
        for (int j = 0; j < values.length; j++) {
           setValue(j, new String(""));
        }        
    }
    
    protected void setODB() {
        amazon = false;
        imdb = false;
    }
}
