/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects.software;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Representation of a Software record. Allows saving, updating and querying
 * of Software records
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Software extends DataCrowObject {

    /** Name of the Software */
    public static final int _A_NAME = 1;
    /** Description */
    public static final int _B_DESCRIPTION = 2;
    /** Webpage */
    public static final int _C_WEBPAGE = 3;
    /** Path */
    public static final int _D_FILENAME = 4;
    /** Software Catagory */
    public static final int _E_TYPE = 5;
    /** Multiplayer Y/N */
    public static final int _F_MULTI = 6;
    /** Location of the Software (for grouping) */
    public static final int _G_LOCATION = 7;

    // hack for the database upgrade(s)
    private static boolean bConvertCatagory = true;

    private static WorkFlow wf;

    /**
     * Initializes the Software object
     */
    public Software() {
       super();
    }

    public static void setConvertCatagory(boolean convert) {
        bConvertCatagory = convert;
    }

    public Object[] getValues() {
        Object[] o = new Object[this.getFieldIndices().length];
        
        int i = 0;
        o[i++] = getValue(_ID);
        o[i++] = getValue(_A_NAME);
        o[i++] = getValue(_B_DESCRIPTION);
        o[i++] = getValue(_C_WEBPAGE);
        o[i++] = getValue(_D_FILENAME);
        o[i++] = getValue(_E_TYPE);
        o[i++] = getValue(_F_MULTI);
        o[i++] = getValue(_G_LOCATION);  
        
        return o;      
    }
    
    public Object getValue(int index) {
        return values[index];   
    }

    /**
     * Applies values to this Software record. The first in the row is the ID
     *
     * @param row   representation of an database row. First in row is the ID
     */
    public void setValues(Object[] row) {
        int i = 0;

        try {
            if (row.length < Repository.dataBase.dbSoftColArray.length) {
                throw new Exception(Resources.getText(DCMessages.msgInvalidArray));
            } else if (row.length == Repository.dataBase.dbSoftColArrayPK.length) {
                if (alwaysGenerateID) {
                    i++; 
                } else {
                    setValue(_ID, row[i++]);
                }
            }

            setValue(_A_NAME, row[i++]);
            setValue(_B_DESCRIPTION, row[i++]);
            setValue(_C_WEBPAGE, row[i++]);
            setValue(_D_FILENAME, row[i++]);
            setValue(_E_TYPE, row[i++]);
            setValue(_F_MULTI, row[i++]);
            setValue(_G_LOCATION, row[i++]);

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Applies a value to this Software object
     *
     * @param iField    field number
     * @param sValue    value String
     */
    public void setValue(int iField, Object value) {
        String sValue = value.toString();
        if (iField != _E_TYPE) {
            super.setValue(iField, sValue);
        } else if (sValue != null && !sValue.trim().equals("") &&
                   bConvertCatagory) {
            try {
                Long.parseLong(sValue);
                sValue = SoftwareCatagories.getCatagoryName(sValue);
            } catch (Exception exp) {
                sValue = SoftwareCatagories.getCatagoryID(sValue);
            }
            super.setValue(iField, sValue);
        } else {
            super.setValue(iField, sValue);
        }
    }
    
    // =========================================================================
    // INITIALIZATION
    // =========================================================================
    protected void setFieldTypes() {
        fieldTypes = Repository.objects.softwareFieldTypes;
    }
    
    protected void setFieldNames() {
        fieldNames = Repository.dataBase.dbSoftColArrayPK;
    }
    
    protected void setLabels() {
        labels = new String[Repository.dataBase.dbSoftColArrayPK.length];
        int i = 0;
        labels[i++] = Resources.getText(DCLabels.lblID);
        labels[i++] = Resources.getText(DCLabels.lblName);
        labels[i++] = Resources.getText(DCLabels.lblDescription);
        labels[i++] = Resources.getText(DCLabels.lblWebPage);
        labels[i++] = Resources.getText(DCLabels.lblFileName);
        labels[i++] = Resources.getText(DCLabels.lblType);
        labels[i++] = Resources.getText(DCLabels.lblMulti);
        labels[i++] = Resources.getText(DCLabels.lblLocation);
    }
    
    protected void setModuleName() {
        moduleName = Resources.getText(DCLabels.lblSoftware);    
    }
    
    protected void setTableName() {
        sTableName = Repository.dataBase.dBSoftTableName.trim();
    }
    
    protected void setRequiredFields() {
        requiredFields = Settings.getValueAsIntegerArray(
                                  Repository.settings.stSoftwareRequiredFields);
    }

    protected void setHiddenFields() {
        hiddenFields = Settings.getValueAsIntegerArray(
                                Repository.settings.stHiddenFieldsSoftware);   
    }
    
    protected void setValueTypes() {
        valueTypes = Repository.objects.softwareValueTypes;
    }
    
    protected void setMaxFieldLengths() {
        maxFieldLength = Repository.objects.softwareFieldLengths;        
    }

    protected void initValues() {
        values = new Object[Repository.dataBase.dbSoftColArrayPK.length];
        for (int j = 0; j < values.length; j++) {
           setValue(j, new String(""));
        }        
    }
    
    protected void setODB() {
        amazon = true;
        imdb = false;
    }    
}
