/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.db;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

import rjw.dc.console.LoginDialog;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowCollection;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.util.Utilities;
import rjw.flex.settings.Settings;

/**
 * Holder of the Database connection and information. Starts the
 * QueryQueue and handles update, insert and select statements.
 * <p>
 * On the first start, the Database will be created (if the database
 * is empty; the tables do not exist). If this fails, a message is
 * shown to the user.
 * <p>
 * For new records, the unique ID is created by this class. This has
 * not been perfected at this moment.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Database {

    private Statement stmt;
    private DatabaseConnection dbConnection;
    private static Connection conn;
    private QueryQueue queue;

    private static boolean bSoftwareTableExist = false;
    private static boolean bCategoryTableExist = false;
    private static boolean bMP3TableExist = false;
    private static boolean bAviTableExist = false;

    /**
     * Initializes the Database, creates a new one if necessary
     */
    public Database() {
        init();
    }

    public static void close() {
        try {
            conn.close();
        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }
    /**
     * Starts the query queue and initializes the database connection
     *
     * @see DatabaseConnection
     * @see QueryQueue
     */
    public void init() {
        boolean bLoadDemoData = Settings.getValueAsBoolean(Repository.settings.stLoadDemoData);
        dbConnection = new DatabaseConnection();

        if (conn == null) {
            initializeDatabase(bLoadDemoData);
        }

        queue = new QueryQueue(conn, stmt);
        Thread queryQueue = new Thread(queue, "queryQueue");

        queryQueue.setPriority(Thread.NORM_PRIORITY);
        queryQueue.start();
    }

    public void insertCollection(DataCrowCollection collection,
                                 RequestorCollection requestors) {
        QueryBatch batch = collection.toQueryBatch(Query._INSERT);
        
        if (batch != null) {
            batch.setRequestors(requestors);        
            queue.addQueryBatch(batch);
        }
    }

    public void updateCollection(DataCrowCollection collection,
                                 RequestorCollection requestors) {
        QueryBatch batch = collection.toQueryBatch(Query._UPDATE);
        
        if (batch != null) {
            batch.setRequestors(requestors);
            queue.addQueryBatch(batch);
        }
    }
    
    public void deleteCollection(DataCrowCollection collection,
                                 RequestorCollection requestors) {
        QueryBatch batch = collection.toQueryBatch(Query._DELETE);
     
        if (batch != null) {
            batch.setRequestors(requestors);   
            queue.addQueryBatch(batch);
        }
    }


    /**
     * Runs a predefined query
     *
     * @param sQuery        query statement
     * @param requestors    requests to the UI
     */
    public void runQuery(String sQuery, RequestorCollection requestors) {
        Query query = new Query(null, null, null, Query._UNDEFINED, requestors, null);
        query.setQueryString(sQuery);
        queue.addQuery(query);
    }

    /**
     * Puts the query in the queue. The queue will handle the query when time
     * comes and returns the results (if there are any) to the requestor by using
     * the RequestorObject class
     *
     * @param query         query object
     * @see RequestorObject
     * @see QueryQueue
     */
    private void putQueryInQueue(Query query) {
        queue.addQuery(query);
    }

    /**
     * Does not put the query in a queue but executes it immediately.
     * Errors are returned to the caller (not thrown)
     *
     * @param sQuery    query statement
     */
    public Object runQueryDirect(String sQuery) {
        ResultSet result = null;

        try {
            result = stmt.executeQuery(sQuery);
        } catch (SQLException sqlExp) {
            if (!sqlExp.getMessage().equals("No ResultSet was produced")) {
                Messages.setMessage(sqlExp.toString(), Messages._ERROR);
            }
            return sqlExp;
        }

        Messages.setMessage(sQuery, Messages._QUERY);

        return result;
    }

    /**
     * Runs a query directly, bypassing the query queue
     *
     * @param DataCrowObject data crow object
     */
    public Object runQueryDirect(DataCrowObject o) {
        QueryWhat queryWhat = new QueryWhat(o.getFieldNames(), o.getTableName());
        Query query = new Query(queryWhat, o.toQueryCondition(true), null, Query._SELECT, null, null);

        Messages.setMessage(query.getQueryString(), Messages._QUERY);
        return runQueryDirect(query.getQueryString());
    }

    /**
     * Does not put the query in a queue but executes it immediately.
     * Errors are returned to the caller (not thrown)
     *
     * @param Query     Query object
     */
    public Object runQueryDirect(Query query) {
        return runQueryDirect(query.getQueryString());
    }

    /**
     * Searches for values. The query is put in the query queue. The results are
     * past to the caller by using the RequestorCollection
     *
     * @param o the object which is used as the condition
     * @param options query options
     * @param requestors requestors for the result of the query (UI)
     */
    public void searchValues(DataCrowObject o, QueryOptions options, RequestorCollection requestors) {
        QueryWhat queryWhat = new QueryWhat(o.getFieldNames(), o.getTableName());
        Query query = new Query(queryWhat, o.toQueryCondition(true), options, Query._SELECT, requestors, null);
        query.setSilence(o.getSilence());

        putQueryInQueue(query);
    }

    /**
     * Updates values. The query is put in the query queue. Actions are
     * past to the caller by using the RequestorCollection
     *
     * @param o            the object to be updated
     * @param requestors    requestors which depend on the outcome of the query
     */
    public void updateValues(DataCrowObject o, RequestorCollection requestors) {
        String sID = o.getValue(DataCrowObject._ID).toString();

        QueryWhat queryWhat = new QueryWhat(o.getFieldNames(), o.getTableName());
        Query query = new Query(queryWhat, o.toQueryCondition(false), null, Query._UPDATE, requestors, sID);
        query.setSilence(o.getSilence());

        putQueryInQueue(query);
    }

    /**
     * Inserts new values. The query is put in the query queue. Actions are
     * past to the caller by using the RequestorCollection
     *
     * @param o   the object to be inserted
     * @param requestors    requestors which depend on the outcome of the query
     */
    public void insertValues(DataCrowObject o, RequestorCollection requestors) {
        String sID = new Utilities().getUniqueID();
        o.setValue(DataCrowObject._ID, sID);

        QueryWhat queryWhat = new QueryWhat(o.getFieldNames(), o.getTableName());
        Query query = new Query(queryWhat, o.toQueryCondition(false), null, Query._INSERT, requestors, sID);
        query.setSilence(o.getSilence());

        putQueryInQueue(query);
    }

    /**
     * Deletes values from the database. The query is put in the query queue. 
     * Actions are past to the caller by using the RequestorCollection
     *
     * @param o the object to be deleted
     * @param requestors requestors which depend on the outcome of the query
     */
    public void deleteValues(DataCrowObject o, RequestorCollection requestors) {
        String sID = o.getValue(DataCrowObject._ID).toString();
        QueryWhat queryWhat = new QueryWhat(o.getFieldNames(), o.getTableName());
        Query query = new Query(queryWhat, o.toQueryCondition(false), null, Query._DELETE, requestors, sID);
        query.setSilence(o.getSilence());

        putQueryInQueue(query);
    }

    /**
     * Checks whether the values are unique (do not exist as yet in the database).
     * Updates and inserts are checked differently
     *
     * @param o   object to be checked
     * @param bUpdateQuery  specifies whether this is an update or an insert
     */
    public boolean uniqueValues(DataCrowObject o, boolean bUpdateQuery) {
        String sID = o.getValue(DataCrowObject._ID).toString();;

        int[]    requiredFields     = o.getRequiredFields();
        String[] conditionFields    = new String[requiredFields.length];
        Object[] conditionValues    = new String[requiredFields.length];

        for (int i = 0; i < requiredFields.length; i++) {
            conditionFields[i] = o.getFieldName(requiredFields[i]);
            conditionValues[i] = o.getValue(requiredFields[i]);
        }

        QueryCondition condition = new QueryCondition(conditionFields, conditionValues, true);
        QueryOptions options = new QueryOptions(null, true, true);
        QueryWhat queryWhat = new QueryWhat(o.getFieldNames(), o.getTableName());
        Query query = new Query(queryWhat, condition, options, Query._SELECT, null, sID);

        ResultSet result = null;
        try {
            result = (ResultSet) runQueryDirect(query.getQueryString());
        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }

        if (result == null) {
            return true;
        }

        int rows = 0;
        try {
            while (result.next()) {
                String recordID = result.getString(Repository.dataBase.dBSoftColPrimary);
                if (!sID.equals(recordID)) {
                    rows++;
                }
            }
        } catch (SQLException sqlExp) {
            Messages.setMessage(sqlExp.toString(), Messages._ERROR);
        }

        if (bUpdateQuery && rows >= 1) {
            return false; // Two rows or more with the same ID
        } else if (!bUpdateQuery && rows > 0) {
            return false; // One row exists already with the same ID
        }
        return true;
    }

    /**
     * Creates a connection to the database. If no database exsists, a new
     * one is created. Demo data is loaded when this is preferred. The login dialog
     * is used to get user input
     *
     * @param loadDemoData  specifies whether to load Demo data upon a new database
     */
    private synchronized void initializeDatabase(boolean loadDemoData) {
        try {
            conn = DatabaseConnection.connect();
            // If Conn is null, the ODBC or the database filename doesn't exsist
            if (conn == null) {
                LoginDialog dialog = new LoginDialog();
                dialog.setVisible(true);                
                while (LoginDialog.done == false) {
                }
                init();
            }

            // Ugly piece of code. Change this in the future!
            // This was done to prevent a Nullpointer as the DB stries to
            // initialize itself twice (strange)
            try {
                stmt = conn.createStatement();
            } catch (Exception sqlExp) {
                return;
            }

            if (stmt.getMaxRows() <= 0) {
                try {
                    String testQuery = "select * from" + Repository.dataBase.dBCatTableName;
                    ResultSet result = stmt.executeQuery(testQuery);
                    initializeColumns(result.getMetaData(), Repository.module._SOFTWARECAT);
                    Messages.setMessage(Resources.getText(DCMessages.msgSoftwareCatTableFound), Messages._STANDARD);
                    bCategoryTableExist = true;
                } catch (SQLException exp) {
                    Messages.setMessage(Resources.getText(DCMessages.msgSoftwareCatTableNotFound), Messages._STANDARD);
                    initializeNewDatabase(loadDemoData, Repository.dataBase.dBCatTableName);
                }

                try {
                    String testQuery = "select * from" + Repository.dataBase.dBMP3TableName;
                    ResultSet result = stmt.executeQuery(testQuery);
                    initializeColumns(result.getMetaData(), Repository.module._MP3);
                    Messages.setMessage(Resources.getText(DCMessages.msgMp3TableFound), Messages._STANDARD);
                    bCategoryTableExist = true;
                } catch (SQLException exp) {
                    Messages.setMessage(Resources.getText(DCMessages.msgMp3TableNotFound), Messages._STANDARD);
                    initializeNewDatabase(loadDemoData, Repository.dataBase.dBMP3TableName);
                }

                try {
                    String testQuery = "select * from" + Repository.dataBase.dBSoftTableName;
                    ResultSet result = stmt.executeQuery(testQuery);
                    initializeColumns(result.getMetaData(), Repository.module._SOFTWARE);
                    Messages.setMessage(Resources.getText(DCMessages.msgSoftwareTableFound), Messages._STANDARD);
                    bSoftwareTableExist = true;
                } catch (SQLException exp) {
                    Messages.setMessage(Resources.getText(DCMessages.msgSoftwareTableNotFound), Messages._STANDARD);
                    initializeNewDatabase(loadDemoData, Repository.dataBase.dBSoftTableName);
                }

                try {
                    String testQuery = "select * from" + Repository.dataBase.dBMovieTableName;
                    ResultSet result = stmt.executeQuery(testQuery);
                    initializeColumns(result.getMetaData(), Repository.module._MOVIE);
                    Messages.setMessage(Resources.getText(DCMessages.msgMovieTableFound), Messages._STANDARD);
                    bAviTableExist = true;
                } catch (SQLException exp) {
                    Messages.setMessage(Resources.getText(DCMessages.msgMovieTableNotFound), Messages._STANDARD);
                    initializeNewDatabase(loadDemoData, Repository.dataBase.dBMovieTableName);
                }
            }
        } catch (SQLException exp) {
            Messages.setMessage(exp.getMessage(), Messages._ERROR);
        }
    }

    private void initializeColumns(ResultSetMetaData metaData, int module) throws SQLException {
        String[] columns = null;
        String[] columnStatements = null;
        String tableName = metaData.getTableName(1);
        
        switch (module) {
            case Repository.module._SOFTWARE:
                columns = Repository.dataBase.dbSoftColArrayPK;
                columnStatements = Repository.dataBase.dbSoftArrayCreate;
                break;
            case Repository.module._SOFTWARECAT:
                columns = Repository.dataBase.dbCatColArrayPK;
                columnStatements = Repository.dataBase.dbCatArrayCreate;
                break;
            case Repository.module._MP3:
                columns = Repository.dataBase.dbMP3ColArrayPK;
                columnStatements = Repository.dataBase.dbMP3ArrayCreate;
                break;
            case Repository.module._MOVIE:
                columns = Repository.dataBase.dbMovieColArrayPK;
                columnStatements = Repository.dataBase.dbMovieArrayCreate;
                break;
        }

        if (metaData == null) {
            // throw error!
            return;
        } 
        
        int metaCount = metaData.getColumnCount();
        if (metaCount < columns.length) {
            Messages.setMessage(Resources.getText(DCMessages.msgTableNeedsUpgrade, tableName),
                                Messages._STANDARD);
            
            String[] metaColumns = new String[metaCount];
            for (int i = 1; i < metaCount + 1; i++) {
                metaColumns[i - 1] = metaData.getColumnName(i);
            }
            
            for (int i = 0; i < columns.length; i++) {
                boolean exists = false;
                for (int j = 0; j < metaColumns.length; j++) {
                    if (columns[i].equalsIgnoreCase(metaColumns[j])) {
                        exists = true;
                    }
                }
                if (!exists) {
                    this.runQueryDirect("alter table " + tableName + " add column " +
                                        columnStatements[i]); 
                }
            }
                    
        }
    }

    /**
     * Creates the tables and fields on an empty database
     *
     * @param loadDemoData  specifies if Demo data should be loaded
     * @param DB            new database name
     */
    private void initializeNewDatabase(boolean loadDemoData, String DB) {
        if (DB.equals(Repository.dataBase.dBSoftTableName)) {
            QueryWhat what = new QueryWhat(Repository.dataBase.dbSoftArrayCreate, Repository.dataBase.dBSoftTableName);
            Query query = new Query(what, null, null, Query._CREATE, null, null);

            this.runQueryDirect(query.getQueryString());
            bSoftwareTableExist = true;

            if (loadDemoData) {
                for (int i = 0; i < Repository.dataBase.dBDemoDataSoftware.length; i++) {
                    this.runQueryDirect(Repository.dataBase.dBDemoDataSoftware[i]);
                }
            }
        }

        if (DB.equals(Repository.dataBase.dBCatTableName)) {
            QueryWhat what = new QueryWhat(Repository.dataBase.dbCatArrayCreate, Repository.dataBase.dBCatTableName);
            Query query = new Query(what, null, null, Query._CREATE, null, null);

            this.runQueryDirect(query.getQueryString());
            bCategoryTableExist = true;

            if (loadDemoData) {
                for (int i = 0; i < Repository.dataBase.dbDemoDataCatagories.length; i++) {
                    this.runQueryDirect(Repository.dataBase.dbDemoDataCatagories[i]);
                }
            }
        }

        if (DB.equals(Repository.dataBase.dBMP3TableName)) {
            QueryWhat what = new QueryWhat(Repository.dataBase.dbMP3ArrayCreate, Repository.dataBase.dBMP3TableName);
            Query query = new Query(what, null, null, Query._CREATE, null, null);

            this.runQueryDirect(query.getQueryString());
            bMP3TableExist = true;
        }

        if (DB.equals(Repository.dataBase.dBMovieTableName)) {
            QueryWhat what = new QueryWhat(Repository.dataBase.dbMovieArrayCreate, Repository.dataBase.dBMovieTableName);
            Query query = new Query(what, null, null, Query._CREATE, null, null);

            this.runQueryDirect(query.getQueryString());
            bMP3TableExist = true;
        }
    }
}
