/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.db;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;

import rjw.dc.console.MessageBox;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**s
 * Query queue with last in last out mechanism. Request for actions depending
 * on the query are past on to the WorkFlow.
 * The queue is in fact a LinkedList
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class QueryQueue extends Thread {

    private LinkedList lQueryQueue = new LinkedList();

    private boolean bLazy = true;
    private static Connection conn;
    private static Statement stmt;

    private static final WorkFlow wf = new WorkFlow();

    /**
     * Initializes the queue, it sets the connection and the statement object
     *
     * @param conn  database connection
     * @param stmt  statement object
     */
    public QueryQueue(Connection connection, Statement statement) {
        conn = connection;
        stmt = statement;
    }

    /**
     * Adds a query to the queue (at the last position)
     *
     * @param query     the query object
     */
    public void addQuery(Query query) {
        lQueryQueue.addLast(query);
    }

    public void addQueryBatch(QueryBatch batch) {
        lQueryQueue.addLast(batch);
    }

    /**
     * Starts the queue. If the queue is empty the thread is set in
     * sleep mode
     */
    public void run() {
        synchronized(this) {
            while (true) {
                try {
                    if (lQueryQueue.size() > 0 && bLazy) {
                        bLazy = false;
                        Object o = lQueryQueue.removeFirst();
                        
                        ResultSet result = null;
                        RequestorCollection requestors = null;
                        if (o instanceof Query) {
                            Query query = (Query) o;
                            result = executeQuery(query);
                            requestors = query.getRequestors();
                        } else if (o instanceof QueryBatch) {
                            QueryBatch batch = (QueryBatch) o;
                            executeQueryBatch(batch);
                            requestors = batch.getRequestors();
                        }
                        
                        updateUI(result, requestors);

                        if (result != null) {
                            result.close();
                        }
                    } else {
                        bLazy = true;
                        sleep(100);
                    }
                } catch (Exception exp) {
                    Messages.setMessage(exp.toString(), Messages._ERROR);
                }
            }
        }
    }

    /**
     * Executes the query
     *
     * @param query         the query object to be executed
     * @return (ResultSet)  a resultset (can be null)
     */
    private ResultSet executeQuery(Query query) {
        switch(query.getQueryType()) {
            case Query._UPDATE:
                executeUpdateQuery(query);
                break;
            case Query._DELETE:
                executeDeleteQuery(query);
                break;                
            case Query._INSERT:
                executeInsertQuery(query);
                break;
            case Query._UNDEFINED:
                Messages.setMessage(query.getQueryString(), Messages._QUERY);
                return executeUndefinedQuery(query);
            default:
                Messages.setMessage(query.getQueryString(), Messages._QUERY);
                return executeSelectQuery(query);
        }

        Messages.setMessage(query.getQueryString(), Messages._QUERY);
        return null;
    }

    private void executeQueryBatch(QueryBatch batch) {
        Query[] queries = batch.getQueries();
        boolean success = false;
        String message = "";
        for (int i = 0; i < queries.length; i++) {
            queries[i].setSilence(true);
            if (queries[i].getQueryType() == Query._UPDATE) {
                success = executeUpdateQuery(queries[i]);
                message = Resources.getText(DCMessages.msgDataSaved);
            } else if (queries[i].getQueryType() == Query._INSERT) {
                success = executeInsertQuery(queries[i]);
                message = Resources.getText(DCMessages.msgDataSaved);
            } else if (queries[i].getQueryType() == Query._DELETE) {
                success = executeDeleteQuery(queries[i]);
                message = Resources.getText(DCMessages.msgDeleteSuccessfull);
            }
            
            Messages.setMessage(queries[i].getQueryString(), Messages._QUERY);      
        }

        if (success) {
            new MessageBox(message, true, MessageBox._INFORMATION, false);      
        }
    }

    /**
     * Executes an update query
     *
     * @param query     the query to be executed
     */
    private boolean executeUpdateQuery(Query query) {
        String sQuery = query.getQueryString();

        try {
            stmt.executeUpdate(sQuery);
            if (!query.getSilence()) {
                new MessageBox( Resources.getText(
                    DCMessages.msgRecSaved, query.getRecordID()), 
                    true, MessageBox._INFORMATION, false);
            }
            return true;
        } catch (SQLException sqlExp) {
            Messages.setMessage(sQuery, Messages._QUERY);
            Messages.setMessage(sqlExp.toString(), Messages._ERROR);
            new MessageBox(Resources.getText(
                DCMessages.msgQueryError, sqlExp.toString()), 
                true, MessageBox._ERROR, false);
            return false;
        }
    }

    /**
     * Executes an insert query
     *
     * @param query     the query to be executed
     */
    private boolean executeInsertQuery(Query query) {
        String sQuery = query.getQueryString();

        try {
            stmt.execute(sQuery);
            if (!query.getSilence()) {
                new MessageBox( 
                    Resources.getText(DCMessages.msgRecSaved, query.getRecordID()), 
                    true, MessageBox._INFORMATION, false);
            }
            return true;            
        } catch (SQLException sqlExp) {
            Messages.setMessage(sQuery, Messages._QUERY);
            Messages.setMessage(sqlExp.toString(), Messages._ERROR);
            new MessageBox(Resources.getText(
                DCMessages.msgQueryError, sqlExp.toString()), 
                true, MessageBox._ERROR, false);
        }
        return false;
    }

    /**
     * Executes a select query
     *
     * @param query         the query to be executed
     * @return (ResultSet)  result of the query
     */
    private boolean executeDeleteQuery(Query query) {
        String sQuery = query.getQueryString();
        try {
            stmt.executeQuery(sQuery);
            if (!query.getSilence()) {
                new MessageBox(
                    Resources.getText(DCMessages.msgRecordDeleted, query.getRecordID()), 
                    true, MessageBox._INFORMATION, false);
            }   
            return true;         
        } catch (SQLException sqlExp) {
            Messages.setMessage(sQuery, Messages._QUERY);
            Messages.setMessage(sqlExp.toString(), Messages._ERROR);
            new MessageBox(
                Resources.getText(DCMessages.msgQueryError, sqlExp.toString()), 
                true, MessageBox._ERROR, false);
        }
        return false;
    }


    /**
     * Executes a select query
     *
     * @param query         the query to be executed
     * @return (ResultSet)  result of the query
     */
    private ResultSet executeSelectQuery(Query query) {
        String sQuery = query.getQueryString();
        ResultSet result = null;

        int iMaxSearcRes = Settings.getValueAsInt(Repository.settings.stMaximumSearchRes);

        try {
            stmt.setMaxRows(iMaxSearcRes);
            result = stmt.executeQuery(sQuery);
        } catch (SQLException sqlExp) {
            Messages.setMessage(sQuery, Messages._QUERY);
            Messages.setMessage(sqlExp.toString(), Messages._ERROR);
            new MessageBox(
                Resources.getText(DCMessages.msgQueryError, sqlExp.toString()), 
                true, MessageBox._ERROR, false);
        }
        return result;
    }

    /**
     * Executes an undified query type (there is no knowledge if a resultset
     * is returned or not)
     *
     * @param query         the query to be executed
     * @return (ResultSet)  result of the query
     */
    private ResultSet executeUndefinedQuery(Query query) {
        String sQuery = query.getQueryString();
        ResultSet result = null;

        int iMaxSearcRes = Settings.getValueAsInt(Repository.settings.stMaximumSearchRes);;

        try {
            stmt.setMaxRows(iMaxSearcRes);
            result = stmt.executeQuery(sQuery);
        } catch (SQLException sqlExp) {
            Messages.setMessage(sQuery, Messages._QUERY);
            Messages.setMessage(sqlExp.toString(), Messages._ERROR);
            new MessageBox(
                Resources.getText(DCMessages.msgQueryError, sqlExp.toString()), 
                true, MessageBox._ERROR, false);
        }
        return result;
    }

    /**
     * Passes requests and data for the UI to the Workflow layer
     *
     * @param result        result of the query
     * @param requestors    requestors for actions or data
     */
    private void updateUI(ResultSet result, RequestorCollection requestors) {
        wf.displayResults(result, requestors);
    }
}
