/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.processes;
import java.io.File;
import java.io.RandomAccessFile;
import java.util.Enumeration;
import java.util.Vector;

import rjw.dc.console.MessageBox;
import rjw.dc.console.software.InsertSoftwarePanel;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.CdRom;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.software.Software;
import rjw.dc.messages.Messages;
import rjw.dc.messages.MessagesWriter;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Exports a Software CD to a file. Later on it can be imported by the ImportFile
 * class. The layout of the export file is like this; value <seperator> value.
 * The seperator is retrieved form the <url>ImportInformationFile</url>
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ExportSoftwareCD extends Thread {

    private CdRom cdRom = new CdRom();
    private String location = "";
    private boolean bConfirmDirs = false;
    private String cdName = "";
    private String file = "";
    public static boolean running = false;
    private static final WorkFlow wf = new WorkFlow();
    private boolean bToTable = false;
    private boolean bToFile = false;

    /**
     * Initializes the export process. All preferences are set
     *
     * @param loc location from which the data is read
     * @param bConfirmDirs indicates if per directory should be asked if the data
     *                     should be read from this directory
     * @param file the export filename
     * @param cdName name of the location, used for grouping data
     */
    public ExportSoftwareCD(String loc,
                            boolean bConfirmDirs,
                            String file,
                            String cdName,
                            boolean bToFile,
                            boolean bToTable) {

        if (!loc.endsWith("\\") && !loc.endsWith("/")) {
            this.location = loc;
        } else {
            this.location = loc.substring(0, loc.length() -1);
        }
        
        this.bConfirmDirs = bConfirmDirs;
        this.file = file;
        this.cdName = cdName;
        this.bToFile = bToFile;
        this.bToTable = bToTable;
    }

    /**
     * Starts the export process. The data read from the location is written
     * to the export file. Information from the files is separated by..
     * seperators as specified in the IIF. Errors are shown to user in
     * messageboxes
     */
    public void run() {
        try {
            running = true;
            Vector vFilesAndDirs = cdRom.getInformation(location, 
                                                        bConfirmDirs, 
                                                        true, 
                                                        true,
                                                        null);
            createOutput(vFilesAndDirs);
        } catch (Exception exp) {
            exp.printStackTrace();
            MessagesWriter._useLogFile = true;
            Messages.setMessage(exp.toString(), Messages._ERROR);
            new MessageBox(Resources.getText(DCMessages.msgProcessCrash), 
                           true, MessageBox._ERROR);
            finishAll();
        }

        new MessageBox(Resources.getText(DCMessages.msgExportCompleted), 
                       true, MessageBox._INFORMATION);

        finishAll();
    }

    /**
     * Writes the data to the file and to the output table.
     * Information from the files is separated by..
     * seperators as specified in the IIF.
     */
    public void createOutput(Vector data) {
        File sourceFile;

        try {
            
            RandomAccessFile fileAccess = null;
            
            new MessageBox( Resources.getText(DCMessages.msgWriteToFile), 
                            true, MessageBox._INFORMATION);            
                    
            if (bToFile) {
                try {
                    sourceFile = new File(file.trim());
                } catch (Exception exp) {
                    sourceFile = new File(Settings.getValueAsString(Repository.settings.stSoftExportFile));
                }
                
                fileAccess = new RandomAccessFile(sourceFile, "rw");
                Messages.setMessage(
                    Resources.getText(DCMessages.msgWritingInfo), 
                    Messages._STANDARD);
    
                fileAccess.seek(fileAccess.length());
                fileAccess.writeBytes("->" + Repository.dataBase.dBSoftTableName.trim() + "<-" + '\n');
            }
            
            boolean first = true;

            for (Enumeration enum = data.elements(); enum.hasMoreElements(); ) {
                String sTempFilename = (String) enum.nextElement();
                String sFilename = sTempFilename.substring(2, sTempFilename.length());

                Messages.setMessage(Resources.getText(DCMessages.msgWritingToFile, sFilename),
                                    Messages._STANDARD);

                if (first && bToFile) {
                    fileAccess.seek(fileAccess.length());

                    fileAccess.writeBytes(sFilename + " > " + 
                                          "    [" + cdName + "]   "
                                          + '\n');
                    first = false;
                } else if (bToFile) {
                    try {
                        sleep(10);
                    } catch (InterruptedException intExp) {}

                    fileAccess.writeBytes(sFilename + " > " + '\n');
                }

                if (bToTable) {
                    Software software = new Software();
                    software.setValue(Software._D_FILENAME, sFilename);
                    software.setValue(Software._G_LOCATION, cdName);
                    Vector vTable = new Vector(0);
                    vTable.add(software);
                    Requestor req = new Requestor(InsertSoftwarePanel.class,
                                                  "addRow",
                                                  null);
                    RequestorCollection reqCol = new RequestorCollection();
                    reqCol.addRequestor(req);
                    wf.displayResults(vTable, reqCol);
                }
            }

            Messages.setMessage(Resources.getText(DCMessages.msgCreatingIIF), Messages._STANDARD);
            String[][] IIFData = { { Resources.getText(DCLabels.lblFileName), ">"}};

            new ImportInformationFile(IIFData, file);
            Messages.setMessage(
                Resources.getText(DCMessages.msgCreatedIIF), 
                Messages._STANDARD);
            
            if (bToFile) {
                fileAccess.close();
            }

        } catch (Exception exp) {
            MessagesWriter._useLogFile = true;
            new MessageBox(
                Resources.getText(DCMessages.msgProcessCrash), 
                true, MessageBox._ERROR);
            Messages.setMessage(exp.toString(), Messages._ERROR);
            finishAll();
        }
    }

    /**
     * Stops the process
     */
    private void finishAll() {
        try {
            running = false;
            MessagesWriter._useLogFile = true;
            Messages.setMessage(
                Resources.getText(DCMessages.msgExportStops), 
                Messages._STANDARD);
        } catch (Exception exp) {
        }
    }
}
