/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.processes;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import rjw.dc.console.MessageBox;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.movie.Movie;
import rjw.dc.core.objects.mp3.Mp3;
import rjw.dc.core.objects.software.Software;
import rjw.dc.messages.Messages;
import rjw.dc.messages.MessagesWriter;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/** Imports data from a external source. Needs to be a textfile!
 *  location:  can be defined like this [NAME] at the end of a row,
 *             OR can be normally defined (like all other fields).
 *             If for a row a location can not be determined the last
 *             used location is used!!
 *  seperator:  The fields end always with a seperator, including the last
 *             one.. like this: name > type > description >
 *             The seperators are retrieved from the <url>ImportInformationFile</url>
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ImporList extends Thread {

    private RandomAccessFile source;
    private File sourceFile;
    private File temp;
    private RandomAccessFile tempFile;
    private String accessMode = "rw";
    private String location = "Null";
    private String sSourceFile = "";
    private long filePointer = 0;
    private static long fileLength;
    private String[] dataFields = null;

    private static int iModule = -1;

    private Requestor requestor;
    private WorkFlow wf = new WorkFlow();

    private ImportInformationFile iff;
    private String[][] seperators;

    /** Mapping info, maps the database fields ([0]) to the IIF data fields ([1]) */
    private int[][] mappingInfo = null;
    /** Indicates whether or not the process is running */
    public static boolean running = false;

    /**
     * Initializes the process, sets the import filename. A requestor
     * is used to write the information to the log
     * @param filename  the import file name (absolute path)
     */
    public ImporList(String fileName) {
        requestor = new Requestor(this.getClass(), "writeToLog", null);
        sSourceFile = fileName;
        temp = new File(Repository.files.flImportTemp);
    }

    /**
     * Starts the actual import of the external source. Data which could be
     * imported is written to a seperated file with a comment containing the
     * error message. After modifying the data, this error file can be used
     * as source for a new import process
     *
     * The data is read by using the IIF
     */
    public void run() {
        sourceFile = new File(sSourceFile);

        iModule = Settings.getValueAsInt(Repository.settings.stModule);
        boolean bRequiredCheck = Settings.getValueAsBoolean(Repository.settings.stCheckRequiredFields);
        boolean bUniquenessCheck = Settings.getValueAsBoolean(Repository.settings.stCheckUniqueness);

        running = true;
        sourceFile = new File(sSourceFile);

        if (sourceFile.exists() == false) {
            // If the source doesn't exsist, exit the method
            Messages.setMessage(
                Resources.getText(DCMessages.msgSourceNotFound), 
                Messages._STANDARD);
            new MessageBox(
                Resources.getText(DCMessages.msgSourceNotFound), 
                true, MessageBox._ERROR);
            running = false;
            return;
        }

        iff = new ImportInformationFile(sSourceFile);
        seperators = iff.getIIFValues();

        try {
            try {
                tempFile = new RandomAccessFile(temp, accessMode);
                source = new RandomAccessFile(sourceFile, accessMode);
            } catch (Exception exp) {
                MessagesWriter._useLogFile = true;
                Messages.setMessage(exp.toString(), Messages._ERROR);
                new MessageBox(
                    Resources.getText(DCMessages.msgInvalidFileName), 
                    true,MessageBox._ERROR);
                finishAll();
            }

            fileLength = source.length();

            // Start reading fom the source file
            boolean firstLine = true;
            while (filePointer < fileLength) {
                try {
                    sleep(50);
                } catch (InterruptedException exp) {
                }
                String line = source.readLine();

                if (firstLine == true) {
                    if (iModule == 0 &&
                        !line.equals("->" + Repository.dataBase.dBSoftTableName.trim() + "<-")) {

                        new MessageBox(Resources.getText(DCMessages.msgInvalidSoftwareImportFile), 
                                       true, MessageBox._ERROR);
                        this.finishAll();
                    } else if (iModule == 1 &&
                               !line.equals("->" + Repository.dataBase.dBMP3TableName.trim() + "<-")) {

                        new MessageBox(Resources.getText(DCMessages.msgInvalidMP3ImportFile), 
                                       true, MessageBox._ERROR);
                        this.finishAll();
                    } else if (iModule == 2 &&
                               !line.equals("->" + Repository.dataBase.dBMovieTableName.trim() + "<-")) {

                        new MessageBox(Resources.getText(DCMessages.msgInvalidMovieImportFile), 
                                       true, MessageBox._ERROR);
                        this.finishAll();
                    }
                }

                if (line != null) {
                    try {
                        // Extract the data from the read line
                        Object[] data = null;
                        try {
                            data = convertLine(line);
                        } catch (Exception exp) {
                            writeToErrorLog(line, exp.getMessage());
                            data = null;
                        }

                        if (data != null) {
                            saveData(data, bRequiredCheck, bUniquenessCheck, line);
                        }

                        filePointer = source.getFilePointer();
                    } catch (Exception exp) {
                        MessagesWriter._useLogFile = true;
                        Messages.setMessage(exp.fillInStackTrace().toString(), Messages._ERROR);
                        new MessageBox(
                            Resources.getText(DCMessages.msgProcessCrash), 
                            true, MessageBox._ERROR);
                        finishAll();
                    }
                } else {
                    filePointer = source.getFilePointer();
                }

                firstLine = false;
            }
        } catch (IOException exp) {
            MessagesWriter._useLogFile = true;
            Messages.setMessage(exp.fillInStackTrace().toString(), Messages._ERROR);
            new MessageBox(
                Resources.getText(DCMessages.msgProcessCrash), 
                true, MessageBox._ERROR);
            finishAll();
        }
        new MessageBox(
            Resources.getText(DCMessages.msgImportCompleted, temp.getName()), 
            true, MessageBox._INFORMATION);
        Messages.setMessage(
            Resources.getText(DCMessages.msgImportStops), 
            Messages._STANDARD);
        finishAll();
    }

    /**
     * Writes an error to the error log
     *
     * @param line                  the failing line
     * @param reason                reason for failing (error string)
     * @exception FileException
     */
    private void writeToErrorLog(String line, String reason) throws IOException {
        tempFile.seek(tempFile.length());
        String ignoredLine = line.trim();

        if (!ignoredLine.equals("")) {
            tempFile.seek(tempFile.length());
            tempFile.writeBytes(line + '\n');
            tempFile.seek(tempFile.length());
            tempFile.writeBytes("@" + Resources.getText(DCMessages.msgReason, reason) + '\n');
        }
    }

    /**
     * Stops the process
     */
    private void finishAll() {
        try {
            MessagesWriter._useLogFile = true;
            running = false;
        } catch (Exception exp) {
        }
    }

    /**
     * Converts the line to a Strign array, which can be used to insert
     * the record into the database. Commentary, lines starting with @,
     * are ignored
     *
     * @param line          line to be converted
     * @return (String[])   data extracted from the line
     * @exception exception
     */
    private Object[] convertLine(String line) throws Exception {
        try {
            sleep(50);
        } catch (InterruptedException exp) {
        }

        if (line.startsWith("@")) {
            // Commentary; ignore!
            return null;
        }
        Object[] data = null;

        if (iModule == 0) {
            data = new Object[Repository.dataBase.dbSoftColArray.length];
        } else if (iModule == 1) {
            data = new Object[Repository.dataBase.dbMP3ColArray.length];
        } else if (iModule == 2) {
            data = new Object[Repository.dataBase.dbMovieColArray.length];
        }

        if (seperators == null) {
            throw new Exception(Resources.getText(DCMessages.msgNoSepException));
        }

        if (dataFields == null) {
            if (iModule == 0) {
                convertIFFtoDB(seperators, Repository.dataBase.dbSoftColArray);
            } else if (iModule == 1) {
                convertIFFtoDB(seperators, Repository.dataBase.dbMP3ColArray);
            } else if (iModule == 2) {
                convertIFFtoDB(seperators, Repository.dataBase.dbMovieColArray);
            }
        }

        int iEnd = 0;
        int iStart = 0;

        for (int i = 0; i < seperators.length; i++) {
            if (seperators[i].length < 2) {
                throw new Exception(Resources.getText(DCMessages.msgNoSepException));
            }
            iEnd = line.indexOf(seperators[i][1]);
            if (iEnd > 0) {
                data[i] = line.substring(iStart, iEnd).trim();
            } else if (iEnd == -1) {
                tempFile.writeBytes(line + '\n');
                throw new Exception(Resources.getText(DCMessages.msgSepNotFound));
            }
            line = line.substring(iEnd + seperators[i][1].length(), line.length());
        }

        data = toStandardFormat(data);

        if (iModule == 0) {
            if (data[6].toString().trim().equals(new String("Null")) ||
                data[6].toString().trim().equals(new String(""))) {

                data[6] = getLocation(line);
            } else {
                location = data[6].toString();
            }
        }

        return data;
    }

    /**
     * Matches the IIF data with the database fields
     *
     * @param iifInformation the iif fields
     * @param colArray the database fields
     */
    private void convertIFFtoDB(String[][] iffInformation, String[] colArray) {
        dataFields = new String[iffInformation.length];
        mappingInfo = new int[iffInformation.length][2];

        for (int i = 0; i < iffInformation.length; i++) {
            if (iffInformation[i][0] != null) {
                for (int j = 0; j < colArray.length; j++) {
                    if (iffInformation[i][0].equalsIgnoreCase(colArray[j])) {
                        dataFields[i] = colArray[j];
                        mappingInfo[i][0] = j;
                        mappingInfo[i][1] = i;
                    }
                }
            }
            if (dataFields[i] == null) {
                dataFields[i] = colArray[i];
                mappingInfo[i][0] = i;
                mappingInfo[i][0] = i;
            }
        }
    }

    /**
     * Saves the data to the database
     *
     * @param data
     * @exception Exception
     */
    private void saveData(Object[] data, boolean bRequiredCheck, boolean bUniquenessCheck, String line) throws Exception {
        DataCrowObject object = null;

        if (iModule == 0) {
            Software software = new Software();
            software.setValues(data);
            object = software;
        } else if (iModule == 1) {
            Mp3 mp3 = new Mp3();
            mp3.setValues(data);
            object = mp3;
        } else if (iModule == 2) {
            Movie movie = new Movie();
            movie.setValues(data);  
            object = movie; 
        }

        object.setValidation(false);

        boolean bUnique = true;
        boolean bRequired = true;
        if (bUniquenessCheck) {
            if (!wf.checkUniqueness(object, false)) {
                bUnique = false;
                writeToErrorLog(line, Resources.getText(DCMessages.msgDataExists));
            }
        }

        if (bRequiredCheck) {
            String sUnfilledFields = object.getUnfilledRequiredFields();
            if (sUnfilledFields.trim().length() > 0) {
                bRequired = false;
                writeToErrorLog(
                    line, 
                    Resources.getText(DCMessages.msgIncompleteRec, sUnfilledFields));
            }
        }

        if (bUnique && bRequired) {
            RequestorCollection requestors = new RequestorCollection();
            requestors.addRequestor(requestor);
            object.setSilence(true);
            object.saveNew(requestors);
        }
    }


    /**
     * Gets the location string of the source file and stores it in location.
     * It returns the begin position of the location.
     * This is used solely in case of the special location specifier; [<location>]
     *
     * @param line      input line
     * @return (String) locationame
     */
    private String getLocation(String line) {
        int locationBegin = line.indexOf(Repository.operators.opImpLocBegin);
        int locationEnd = line.indexOf(Repository.operators.opImpLocEnd);
        if ((locationBegin > -1 && locationEnd > -1) && locationBegin < locationEnd) {
            location = line.substring(locationBegin + 1, locationEnd);
        }
        return location;
    }

    /**
     * Sets the data in the right order for the insert query
     *
     * @param data          data to be converted
     * @return (String[])   converted data
     */
    private Object[] toStandardFormat(Object[] data) {
        Object[] newData = null;

        if (iModule == 0) {
            newData = new Object[Repository.dataBase.dbSoftColArray.length];
        } else if (iModule == 1) {
            newData = new Object[Repository.dataBase.dbMP3ColArray.length];
        } else if (iModule == 2) {
            newData = new Object[Repository.dataBase.dbMovieColArray.length];
        }

        for (int j = 0; j < newData.length; j++) {
            newData[j] = "Null";
        }

        for (int i = 0; i < mappingInfo.length; i++) {
            newData[mappingInfo[i][0]] = data[mappingInfo[i][1]];
        }

        return newData;
    }
}
