/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.processes;
import java.io.File;
import java.io.RandomAccessFile;
import java.util.Vector;

import rjw.dc.core.Repository;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;

/**
 * Import Information File. Needed to create a mapping between the source
 * (a text file) and the database fieldnames. This class creates and reads
 * IIF data and files. The format of the IIF data is like this:
 * <fieldname> <seperator>
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ImportInformationFile {

	private static RandomAccessFile iifAccess;

    /**
     * Creates a new Import Information File
     *
     * @param sourceFileName    name of the import file. The filename will
     *                          be converted to: <sourcefilename>.iif
     */
	public ImportInformationFile(String sourceFileName) {
		try {
			iifAccess = new RandomAccessFile(getIIFFile(sourceFileName), "rw");
		} catch (Exception exp) {
			Messages.setMessage(
				Resources.getText(DCMessages.msgIIFReadError),
				Messages._ERROR);
			Messages.setMessage(exp.getMessage(), Messages._ERROR);
		}
	}

	/**
     * Creates a new IIF file. First in the array is the Database field,
     * second is the seperator value
     *
     * @param seperatorValues   seperator definitions
     * @param sourceFileName    name of the import file The filename will
     *                          be converted to: <sourcefilename>.iif
     */
	public ImportInformationFile(String[][] seperatorValues, String sourceFileName) {
		String line = "";

		try {
			File iifFile = getIIFFile(sourceFileName);
			if (iifFile != null) {
                RandomAccessFile access = new RandomAccessFile(iifFile, "rw");
                access.setLength(0l);
                access.close();
				iifFile = getIIFFile(sourceFileName);
			}

			iifAccess = new RandomAccessFile(getIIFFile(sourceFileName), "rw");

			for (int i = 0; i < seperatorValues.length; i++) {
				for (int j = 0; j < seperatorValues[i].length; j++) {
					if (seperatorValues[i][j] == null) {
						seperatorValues[i][j] = "";
					}
					int test = seperatorValues[i].length - j;
					if (test > 1) {
						line =
							line
								+ seperatorValues[i][j]
								+ Repository.operators.opIIFsep;
					} else {
						line =
							line
								+ seperatorValues[i][j]
								+ Repository.operators.opIIFsep
								+ "\n";
					}
				}
				iifAccess.writeBytes(line);
				line = "";
			}
            iifAccess.close();
		} catch (Exception exp) {
			Messages.setMessage(
				Resources.getText(DCMessages.msgIIFError),
				Messages._ERROR);
			Messages.setMessage(exp.getMessage(), Messages._ERROR);
		}
	}

    /**
     * Returns the IIF data. The return format is as follows:
     * [x][0] = fieldname, [x][1] = seperator value
     *
     * @return (String[][]) the IIF data
     */
	public String[][] getIIFValues() {
		Vector vect = new Vector();
		String[][] sepData = null;

		Messages.setMessage(
			Resources.getText(DCMessages.msgIIFGetInfo),
			Messages._STANDARD);

		try {
			long length = iifAccess.length();
			long filePointer = 0;
			while (filePointer < length) {
				String line = iifAccess.readLine();
				vect.addElement(line);
				filePointer = iifAccess.getFilePointer();
			}

			sepData = new String[vect.size()][10];

			for (int i = 0; i < vect.size(); i++) {
				boolean bFound = true;
				String line = vect.elementAt(i).toString();
				int iStart = 0;
				int counter = 0;
				while (bFound == true) {
					int iEnd = line.indexOf(Repository.operators.opIIFsep);
					if (iEnd >= 0) {
						sepData[i][counter] = line.substring(iStart, iEnd);
						line =
							line
								.substring(
									iEnd + Repository.operators.opIIFsep.length(),
									line.length())
								.trim();
					} else if (iEnd == -1) {
						bFound = false;
					}
					counter++;
				}
			}
		} catch (Exception exp) {
			Messages.setMessage(
				Resources.getText(DCMessages.msgIIFReadError),
				Messages._ERROR);
			Messages.setMessage(exp.getMessage(), Messages._ERROR);
		}

		Messages.setMessage(
			Resources.getText(DCMessages.msgIIFInfoGotten),
			Messages._STANDARD);

		return cleanUp(sepData);
	}

    /**
     * Returns the IIF for the specified import file
     *
     * @param sourceFileName    name of the import file
     * @return (File)           the IIF file
     */
	private File getIIFFile(String sourceFileName) throws Exception {
		String newFileName = "";

		int pos = sourceFileName.indexOf(".");

		if (pos > 0) {
			newFileName = sourceFileName.substring(0, pos);
			newFileName = newFileName + Repository.files.flIIFExtension;
		} else {
			newFileName = sourceFileName + Repository.files.flIIFExtension;
		}

		File iifFile = new File(newFileName);

		return iifFile;
	}

    /**
     * Cleans the IIF data from null values
     *
     * @param data              data to be cleaned
     * @return (String[][])     cleaned data, no null values
     */
	private String[][] cleanUp(String[][] data) {
		int counter = 0;
		String[][] newData = null;

		try {
			for (int j = 0; j < data[0].length; j++) {
				if (data[0][j] != null) {
					counter++;
				}
			}

			newData = new String[data.length][counter];

			for (int k = 0; k < newData.length; k++) {
				for (int l = 0; l < newData[k].length; l++) {
					newData[k][l] = data[k][l];
				}
			}
		} catch (Exception exp) {
			return data;
		}

		return newData;
	}
}
