/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.wf;

import java.sql.ResultSet;
import java.util.Vector;

import javax.swing.SwingUtilities;

import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowCollection;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.movie.Movie;
import rjw.dc.core.objects.mp3.Mp3;
import rjw.dc.core.objects.software.SoftwareCatagory;
import rjw.dc.core.objects.software.Software;
import rjw.dc.db.Database;
import rjw.dc.db.Query;
import rjw.dc.db.QueryOptions;
import rjw.dc.messages.Messages;

/**
 * This is the layer between the UI and the database. Checks for data
 * integrity and checks on bussiness rules are done here
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class WorkFlow {

    private static Database db;

    public static DataCrowObject getObject(int index) {
        switch (index) {
            case Repository.module._SOFTWARE:
                return new Software();
            case Repository.module._MP3:
                return new Mp3();
            case Repository.module._MOVIE:
                return new Movie();
            case Repository.module._SOFTWARECAT:
                return new SoftwareCatagory();
        }
        return null;        
    }


    /**
     * Initializes the database
     */
    public static void initDatabase() {
        if (db == null) {
            db = new Database();
        }
    }

    public void updateCollection(DataCrowCollection collection,
                                 RequestorCollection requestors) {
        initDatabase();
        db.updateCollection(collection, requestors);
    }

    public void insertCollection(DataCrowCollection collection,
                                 RequestorCollection requestors) {
        initDatabase();
        db.insertCollection(collection, requestors);
    }

    public void deleteCollection(DataCrowCollection collection,
                                 RequestorCollection requestors) {
        initDatabase();
        db.deleteCollection(collection, requestors);
    }

    /**
     * Runs a predefined query
     *
     * @param sQuery        query statement
     * @param requestors    requests to the UI
     */
    public void runQuery(String sQuery, RequestorCollection requestors) {
        initDatabase();
        db.runQuery(sQuery, requestors);
    }

    /**
     * Searches for values. The query is put in the query queue. The results are
     * past to the caller by using the RequestorCollection
     *
     * @param o   the object which is used as the condition
     * @param options       query options
     * @param requestors    requestors for the result of the query (UI)
     */
    public void searchValues(DataCrowObject o, QueryOptions queryOptions, RequestorCollection requestors) {
        initDatabase();
        db.searchValues(o, queryOptions, requestors);
    }

    /**
     * Inserts new values. The query is put in the query queue. Actions are
     * past to the caller by using the RequestorCollection
     *
     * @param o   the object to be inserted
     * @param requestors    requestors which depend on the outcome of the query
     */
    public void insertValues(DataCrowObject o, RequestorCollection requestors) {
        initDatabase();
        db.insertValues(o, requestors);
    }

    /**
     * Updates values. The query is put in the query queue. Actions are
     * past to the caller by using the RequestorCollection
     *
     * @param o   the object to be updated
     * @param requestors    requestors which depend on the outcome of the query
     */
    public void updateValues(DataCrowObject o, RequestorCollection requestors) {
        initDatabase();
        db.updateValues(o, requestors);
    }

    public void deleteValues(DataCrowObject o, RequestorCollection requestors) {
        initDatabase();
        db.deleteValues(o, requestors);
    }

    /**
     * Runs a query directly, bypassing the query queue
     *
     * @param o  Data Crow object
     */
    public Object runQueryDirect(DataCrowObject o) {
        initDatabase();
        return db.runQueryDirect(o);
    }

    /**
     * Does not put the query in a queue but executes it immediately.
     * Errors are returned to the caller (not thrown)
     *
     * @param Query     Query object
     */
    public Object runQueryDirect(Query query) {
        initDatabase();
        return db.runQueryDirect(query);
    }

    /**
     * Does not put the query in a queue but executes it immediately.
     * Errors are returned to the caller (not thrown)
     *
     * @param sQuery    query statement
     */
    public Object runQueryDirect(String sQuery) {
        initDatabase();
        return db.runQueryDirect(sQuery);
    }

    /**
     * Passes requests thru to the UI (<url>MainFrame</url>)
     *
     * @param vTable data to be shown
     * @param requestorObjects collection of requests
     */
    public void displayResults(Vector vTable, RequestorCollection requestorObjects) {
        final Vector vValues = vTable;
        final RequestorCollection reqObjects = requestorObjects;

        Runnable runMe = new Runnable() {
            public void run() {
                if (DataCrow.mainFrame == null) {
                    return;
                } else if (vValues != null) {
                    DataCrow.mainFrame.displayResults(reqObjects, vValues);
                }
            }};

        SwingUtilities.invokeLater(runMe);
    }

    /**
     * Passes requests thru to the UI (<url>MainFrame</url>)
     *
     * @param result            the result of the query
     * @param requestorObjects  collection of requests
     */
    public void displayResults(ResultSet result, RequestorCollection requestorObjects) {
        final Vector vTable = createDataCrowObjectArray(result);
        displayResults(vTable, requestorObjects);
    }

    /**
     * Checks if the object is unique
     *
     * @param 0                 object to be checked
     * @param bUpdate           specifies if the object exists already in the
     *                          database or not
     * @return (boolean)        uniqueness identifier
     */
    public boolean checkUniqueness(DataCrowObject o, boolean bUpdate) {
        initDatabase();
        return db.uniqueValues(o, bUpdate);
    }

    /**
     * Converts a resultset to a DataCrowObject Vector
     *
     * @param result        raw result from a query
     * @return (Vector)     vector of Data Crow
     *
     */
    public static Vector createDataCrowObjectArray(ResultSet result) {
        Vector returnValues = new Vector(0);

        if (result == null) {
            return returnValues;
        }

        try {
            String sTableName = result.getMetaData().getTableName(1);
            int columns = result.getMetaData().getColumnCount();
            String[] sColumns = new String[columns];

            int k = 0;
            for (int i = 1; i < columns + 1; i++) {
                sColumns[k] = result.getMetaData().getColumnName(i);
                k++;
            }

            while (result.next()) {
                Object[] rowData = new String[columns];
                
                for (int j = 0; j < columns; j++) {
                    Object value = result.getString(sColumns[j]);
                    rowData[j] = value == null ? "" : value;
                }

                if (    sTableName.toUpperCase().equals(
                        Repository.dataBase.dBSoftTableName.toUpperCase().trim())) {
                    Software software = new Software();
                    software.setValues(rowData);
                    returnValues.add(software);
                } else if ( sTableName.toUpperCase().equals(
                            Repository.dataBase.dBCatTableName.toUpperCase().trim())) {
                    SoftwareCatagory catagory = new SoftwareCatagory();
                    catagory.setValues(rowData);
                    returnValues.add(catagory);
                } else if (sTableName.toUpperCase().equals(
                        Repository.dataBase.dBMP3TableName.toUpperCase().trim())) {
                    Mp3 mp3 = new Mp3();
                    mp3.setValues(rowData);
                    returnValues.add(mp3);
                } else if ( sTableName.toUpperCase().equals(
                            Repository.dataBase.dBMovieTableName.toUpperCase().trim())) {
                    Movie movie = new Movie();
                    movie.setValues(rowData);
                    returnValues.add(movie);
                }
            }
        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
        return returnValues;
    }
}
